#include <stdlib.h>
#include <pthread.h>
#include <leitm.h>
#include "ui.h"

/* variables of the main program */
extern float slice_dist_fact;

extern float cloud_alpha;
extern float cloud_cover;
extern float cloud_sharpness;
extern float cloud_scale;
extern float sun_phi, sun_theta, sun_size;
extern int animate, animate_clouds;
extern float cloud_move_speed, cloud_anim_speed;
extern int cloudmap_invalid;


static void bn_handler(Widget w, void *cdata, void *cbs);
static void scale_handler(Widget w, void *cdata, void *cbs);

static lmwidget *win;
static lmwidget *scale_cover, *scale_sharp, *scale_scale;
static lmwidget *bn_exit, *bn_recalc;
lmwidget *pbar_calc;

int init_gui(int *argc, char **argv)
{
	lmwidget *rows, *hbox;

	if(!(win = lm_init_args(argc, argv))) {
		return -1;
	}

	rows = lm_vbox(win);

	lm_label(rows, "slice dist (needs noise recalculation!)");
	lm_fltscale_link(rows, LM_HORIZONTAL, 1, 0.0, 5.0, slice_dist_fact, &slice_dist_fact);
	bn_recalc = lm_button_call(rows, "recalculate noise", bn_handler);
	/*pbar_calc = lm_progress(rows);*/

	lm_separator(rows, LM_HORIZONTAL);

	lm_label(rows, "tex-coord scale");
	scale_scale = lm_fltscale_link(rows, LM_HORIZONTAL, 1, 0.0, 2.0, cloud_scale, &cloud_scale);

	lm_label(rows, "coverage");
	scale_cover = lm_fltscale_call(rows, LM_HORIZONTAL, 1, 0.0, 1.0, cloud_cover, scale_handler);
	
	lm_label(rows, "sharpness");
	scale_sharp = lm_fltscale_call(rows, LM_HORIZONTAL, 1, 0.0, 1.0, cloud_sharpness, scale_handler);

	lm_separator(rows, LM_HORIZONTAL);

	lm_check_link(rows, "animate texture", animate, &animate);
	lm_fltscale_link(rows, LM_HORIZONTAL, 1, 0.0, 0.5, cloud_move_speed, &cloud_move_speed);
	lm_check_link(rows, "animate clouds", animate_clouds, &animate_clouds);
	lm_fltscale_link(rows, LM_HORIZONTAL, 1, 0.0, 1.0, cloud_anim_speed, &cloud_anim_speed);

	lm_separator(rows, LM_HORIZONTAL);

	lm_label(rows, "sun size");
	lm_fltscale_link(rows, LM_HORIZONTAL, 0, 0.0, 1000.0, sun_size, &sun_size);
	
	lm_label(rows, "sun theta");
	lm_fltscale_link(rows, LM_HORIZONTAL, 1, 0.0, 360.0, sun_theta, &sun_theta);

	lm_label(rows, "sun phi");
	lm_fltscale_link(rows, LM_HORIZONTAL, 1, -90.0, 90.0, sun_phi, &sun_phi);

	lm_separator(rows, LM_HORIZONTAL);
	
	bn_exit = lm_button_call(rows, "quit", bn_handler);
	
	return 0;
}

static void bn_handler(Widget w, void *cdata, void *cbs)
{
	if(cdata == bn_exit) {
		exit(0);
	} else if(cdata == bn_recalc) {
		void invalidate_cache(void);
		invalidate_cache();
		cloudmap_invalid = 1;
	}
}

static void scale_handler(Widget w, void *cdata, void *cbs)
{
	float *ptr = 0;
	if(cdata == scale_cover) {
		ptr = &cloud_cover;
	} else if(cdata == scale_sharp) {
		ptr = &cloud_sharpness;
	} else if(cdata == scale_scale) {
		ptr = &cloud_scale;
	} else {
		return;
	}

	*ptr = lm_fltscale_get_value((lmwidget*)cdata);
	cloudmap_invalid = 1;
}

static void *thread_func(void *arg)
{
	lm_show(win);
	lm_main_loop();
	return 0;
}

void run_gui(void)
{
	pthread_t gui_thread;

	pthread_create(&gui_thread, 0, thread_func, 0);
}
